if (window.location.pathname.endsWith("/")) window.location.pathname = window.location.pathname.substring(0, window.location.pathname.length - 1)

Console.parse = function(result) {
	var r = JSON.parse(result);
	var line = r['line 0'];
	return line ? JSON.parse(line) : r;
}

Console.command = function(command) {
	return Remote.get("/console/command/" + command)
		.then(function(response) {
			var result = Console.parse(response);
			return Promise.resolve(result);
		});
}

var RPC = (function(remote, bwconsole) {

	var paths = {
		plataforma : 	{server : 'backend', path :'/ops/rpc/PlatformAdminApi'},
		clientApi : 	{server : 'backend', path :'/rpc/ClientApi'},
		ops2 : 			{server : 'backend', path :'/ops/rpc/OpsApi'},
		slayer : 		{server : 'slayer', path :'/slayer/rpc' /*'/slayer/rpc'*/}
	};

	var request = function(path, method, params, use_auth) {
		var extractResult = function(response) {
			var result = JSON.parse(response);
			return result.error ? Promise.reject(result.error) : Promise.resolve(result.result);
		}

		var fixPathInfo = function(info) {
			// change qa balancer for qa specific server
			var surl = info[path.server];
			surl = (surl == "http://bubblewitch2qa.midasplayer.com:80") ? "http://bubblewitch2-01.qa.midasplayer.com:8008" : surl;
			surl = (surl == "http://igpqa.midasplayer.com:80") ? "http://charlie.qa.midasplayer.com:8008" : surl;
			surl = (surl.endsWith("08")) ? surl : surl + '08'
			return surl;
		};

		return bwconsole.command('info')
			.then(function(info) {
				return new Promise(function(resolve) {
					var server = fixPathInfo(info)
					var url = server + path.path;
					var content = {"method" : method, "params" : params, "id" : 0};

					if (typeof use_auth === 'boolean' && use_auth) {
						getSessionKey(info.coreuserid)
							.then(function(session) {
								url += '?_session=' + session;
								content.sessionKey = session;
								remote.post(url, content)
									.then(extractResult)
									.then(resolve);
							});
						
					} else {
						remote.post(url, content)
							.then(extractResult)
							.then(resolve);
					}
				});
			})
			.catch(function(error) {
				alert(error);
			});
	}


	var getSessionKey = function(coreuserid) {
		var method = 'OpsSessionApi.getSessionKey';
		var params = [parseInt(coreuserid), {id:5, name:'backend'}];

		return request(paths.ops2, method, params);
	}

	var setProgression = function(level) {
		var method = 'JuegoResetApi.goToLevel';
		var params = [parseInt(level)];

		return request(paths.clientApi, method, params, true);
	}

	var resetProgression = function() {
		var method = 'JuegoResetApi.resetLevelProgression';
		var params = [];

		return request(paths.clientApi, method, params, true);
	}

	var setHardCurrency = function(amount) {
		var method = 'JuegoResetApi.setHardCurrency';
		var params = [parseInt(amount)];

		return request(paths.clientApi, method, params, true);
	}

	var addLiveOpsEvent = function(liveOpsJson) {
		var method = 'LiveOpsEventsAdminApi.addEvent';
		var params = [liveOpsJson];

		return request(paths.plataforma, method, params);
	}

	var getAdminEvent = function(liveOpsId) {
		var method = 'LiveOpsEventsAdminApi.getAdminEvent';
		var params = [liveOpsId];

		return request(paths.plataforma, method, params);
	}

	var setSLayerWhiteList = function(coreuserid, messageId) {
		var method = 'ToolsProviderApi.setWhitelistForUser';
		var params = [parseInt(coreuserid), {"deviceName":"WebConsole Test","messageIds":[messageId]}];

		return request(paths.slayer, method, params);
	}

	return {
		request : request,
		getSessionKey : getSessionKey,
		setProgression : setProgression,
		resetProgression : resetProgression,
		setHardCurrency : setHardCurrency,
		addLiveOpsEvent : addLiveOpsEvent,
		getAdminEvent : getAdminEvent,
		setSLayerWhiteList : setSLayerWhiteList	}
})(Remote, Console);


var Performance = (function(bwconsole, window) {

	var template = function(data) {
		return `<div title="cpu"><label>cpu:</label>${data.cpu}</div>
				<div title="memory"><label>mem:</label>${data.memory}</div>
				<div title="time per frame"><label>tpf:</label>${data.timeperframe}</div>
				<div title="frames per second"><label>fps:</label>${data.fps}</div>
				<div title="render calls"><label>rcalls:</label>${data.rendercalls}</div>`;
	}

	var interval;

	var requestPerformance = function() {
		bwconsole.command('performance')
			.then(function(perf) {
				G('performanceDisplay').innerHTML = template(perf);
			});
	}

	var start = function(freq, display) {
		stop();

		var f = (freq ? parseInt(freq) : 5) * 1000;
		requestPerformance();
		interval = window.setInterval(requestPerformance, f);
		G('performanceDisplay').style.display = "block";
	}

	var stop = function() {
		window.clearInterval(interval);
		G('performanceDisplay').style.display = "none";
	}

	return {
		start: start,
		stop: stop,
	}
})(Console, window);


var LiveOps = (function(rpc, bwconsole) {

	var template = {"type":1,"scope":1,"targetType":5,"validFrom":1455641639,"validTo":1455667500,"teaserTime":1455641639,"targets":[{"targetOrder":1,"targetAmount":"1","canvasRewards":[],"mobileRewards":[{"itemType":11018,"itemQuantity":"5"}]}],"minimumLevel":0,"minimumClientAppVersion":"1.2.3","additionalInfo":"","platform":1,"collectBubblesExtraFields":{"progressCap":null},"jackalopeExtraFields":{"levelsBundle":"01JACKALOPE-122","theme":"test-4"},"pullTheRopeExtraFields":{"progressCap":null,"teams":2,"zoomedInIntervalSize":0.1},"goldenBubblesExtraFields":{}};

	var types = {	1 : {id: 1, name: "collectibles", extraName: "collectBubblesExtraFields", extraFields : ["progressCap"]}, 
					2 : {id: 2, name: "jackalope", extraName: "jackalopeExtraFields", extraFields : ["levelsBundle", "theme"]}, 
				  	3 : {id: 3, name : "pulltherope", extraName: "pullTheRopeExtraFields", extraFields : ["teams", "progressCap", "zoomedInIntervalSize"]},
				  	4 : {id: 4, name: "goldenbubbles", extraName: "goldenBubblesExtraFields", extraFields : []}
				};


	var retrieveFormOptions = function() {
		var liveop = types[G('liveOpsType').value];

		var teaserTime = parseInt(G('teaserTime').value);
		var teaserDuration = parseInt(G('teaserDuration').value);
		var liveopsDuration = parseInt(G('liveopsDuration').value);

		var now = Math.floor(new Date().getTime() / 1000);
		var teaserStarts = now + teaserTime;
		var liveopsStarts = teaserStarts + teaserDuration;
		var liveopsEnds = liveopsStarts + liveopsDuration;

		return {
			"type": liveop.id,
			"teaserTime": teaserStarts,
		    "validFrom": liveopsStarts,
		    "validTo": liveopsEnds
		}
	}

	var retrieveFormExtendedOptions = function() {
		var minLevel = parseInt(G('minLevel').value);
		var liveopsRewardProgression = parseInt(G('liveopsRewardProgression').value);

		var mobileRewards = [];
		var entries =  document.getElementsByName('liveopsRewardEntry');
		for (var i = 0; i < entries.length; i++) {
			var entry = entries[i];
			mobileRewards[mobileRewards.length] = {
				"itemType": parseInt(entry.querySelector("select[id=liveopsRewardType]").value),
				"itemQuantity": parseInt(entry.querySelector("input[id=liveopsRewardQuantity]").value)
			};
		}

		return {
			"minimumLevel": minLevel,
			"targets": [
			    {
			    	"targetOrder": 1,
			    	"targetAmount": liveopsRewardProgression,
			    	"canvasRewards": [],
			    	"mobileRewards": mobileRewards
				}
			]
		}
	}

	var create = function(extra) {
		var liveOpsJson = Object.assign(template, retrieveFormOptions(), retrieveFormExtendedOptions(), extra);
		
		rpc.addLiveOpsEvent(liveOpsJson)
			.then(getSLayerMessageId)
			.then(addMessageToWhitelist)
			.then(forceServiceLayerSync)
			.then(function() {alert('LiveOps created!');});


	}

	var getSLayerMessageId = function(result) {
		return rpc.getAdminEvent(result.responseDTO)
			.then(function(response) {
				return new Promise(function(resolve) {
					var data = response;
					var liveopData = types[data.responseDTO.type];
					if (liveopData) {
						var slayerId = data.responseDTO[liveopData.extraName].slayerMessageId;
						resolve(slayerId);
					} else {
						resolve(0);
					}
				});
			});
	}

	var addMessageToWhitelist = function(messageId) {
		return new Promise(function(resolve, reject) {
			if (messageId > 0) {
				bwconsole.command('info')
					.then(function(info) {
						return rpc.setSLayerWhiteList(info.coreuserid, messageId);
					})
					.then(resolve);
			} else {
				resolve(0);
			}
			
		});
	}

	var forceServiceLayerSync = function() {
		return bwconsole.command("sync");
	}

	return {
		create : create
	}

})(RPC, Console);


var HardCurrency = (function(rpc, bwconsole) {

	var set = function(amount) {
		RPC.setHardCurrency(amount)
		.then(function() {
			bwconsole.request("sync");
		});
	}
	
	return {
		set : set
	}

})(RPC, Console);


var Progression = (function(rpc, bwconsole) {

	var reset = function() {
		
		bwconsole.command('go menu')		
			.then(function() {
				window.setTimeout(function() {bwconsole.command('level reset')}, 100);
				window.setTimeout(function() {bwconsole.command('go map')}, 200);
			})
	}

	var set = function(level) {
		rpc.setProgression(level)
			.then(function() {
				sync(level);
			});
		
	}

	var last = function() {
		bwconsole.command("level last")
			.then(function(response) {
				var lastlevel = bwconsole.parse(response);
				set(parseInt(lastlevel));
			});
	}

	var sync = function(level) {
		bwconsole.command('go menu')
			.then(function() {
				window.setTimeout(function() {
					bwconsole.command('go map')
					.then(function() {if (level) bwconsole.command('level center ' + level)});	
				}, 500);
			});
		
	}
	
	return {
		set : set,
		last : last,
		reset : reset,
		sync : sync
	}

})(RPC, Console);

var Reward = (function(bwconsole) {
	var create = function(baseId) {
		var rewards = [];
		var entries =  document.getElementsByName('rewardEntry');
		for (var i = 0; i < entries.length; i++) {
			var entry = entries[i];
			rewards.push(entry.querySelector("select[id=rewardElement]").value);
			rewards.push(entry.querySelector("input[id=rewardQuantity]").value);
		}

		var command = 'reward ' + G('rewardType').value + ' ' + rewards.join(' ');
		bwconsole.command(command);
	};

	return {
		create : create
	}
})(Console);

var UI = (function() {
	var addRewardLine = function(baseId) {
		var base = G(baseId);
		var cloned = base.cloneNode(true);
		cloned.removeAttribute('id');
		cloned.classList.remove('first');

		base.parentElement.appendChild(cloned);
	};

	var info = function(action) {
		G('infoDisplay').style.display = (action == 'hide' ? 'none' : 'block');
	};

	return {
		addRewardLine : addRewardLine,
		info : info
	}
})();

var CommandExecutor = (function(bwconsole, window) {

	var parse = function(text) {
		var commands = text.split(/\r?\n/);
		console.log(commands);
		return commands;
	}

	var execute = function(commands) {
		var sequence = Promise.resolve();

		commands = Object.prototype.toString.call( commands ) === '[object Array]' ? commands : [commands];
		for (var i = 0; i < commands.length; i++) {
			var command = commands[i];
			if (/^wait /.test(command)) {
				sequence = sequence.then(createCommandCheckPromise(command.replace("wait ", "")));
			} else {
				sequence = sequence.then(createCommandPromise(command));	
			}
		}
		sequence.catch(function(reason){console.error(reason)});
	}

	var createCommandPromise = function(command, delay) {
		return function() {
			delay = delay || 1000;
			return new Promise(function(resolve) {
				console.log(command);
				bwconsole.request(command, function() {
					window.setTimeout(resolve, delay);		
				});
			});
		}
	}

	var createCommandCheckPromise = function(command, timeout, pollFreq) {
		return function() {
			pollFreq = pollFreq || 1  * 1000;
			timeout  = timeout  || 10 * 1000;

			return new Promise(function(resolve, reject) {
				var call = function() {
					console.log(command);
					bwconsole.request(command, function(response) {
						var success = bwconsole.parse(response).found;
						if (success) {
							window.clearTimeout(timeouting);
							window.clearInterval(polling);
							resolve();
						}	
					});
				}

				var cancel = function() {
					window.clearInterval(polling);
					reject("timeout for " + command);
				}

				var timeouting = window.setTimeout(cancel, timeout);
				var polling = window.setInterval(call, pollFreq);
				call();
			});	
		}	
	}

	return {
		execute : execute
	}

})(Console, window);


var Tests = (function(executor) {

	var execute = function(name) {
		var commands = ["lives infinite 7200"];
		for (var i = 100; i <= 103; i++) {
			commands[commands.length] = "go map";
			commands[commands.length] = "level open " + i;
			commands[commands.length] = "wait so shooter";
			commands[commands.length] = "popup close";
			commands[commands.length] = "game win";
		}
		executor.execute(commands);
	}

	return {
		execute : execute
	}
})(CommandExecutor);


window.addEventListener('load', function() {

	// load custom html file
	CustomContent.load('console/console-custom.html', 'console-custom', function(text) {

		// load info
		Console.command('info')
			.then(function(info) {

				var template = function(data) {
					return `<div title="backend"><label>backend:</label>${info.backend}</div>
							<div title="Service Layer"><label>slayer:</label>${info.slayer}</div>
							<div title="coreuserid"><label>coreuserid:</label>${info.coreuserid}</div>
							<div title="last level"><label>last level:</label>${info['last-level']}</div>
							<div title="last episode"><label>last ep:</label>${info['last-episode']}</div>`;
				}

				G('infoDisplay').innerHTML = template(info);
			});
	});

});
